@@ -9,6 +9,7 @@ from ConfigParser import ConfigParser
 from itertools import islice
 import tarfile
 from shutil import rmtree
+from zipfile import ZipFile, ZIP_DEFLATED
 
 import tg
 from pylons import app_globals as g
@@ -66,7 +67,12 @@ class HgImplementation(M.RepositoryImplementation):
 
     @LazyProperty
     def _hg(self):
-        return hg.repository(HgUI(), self._repo.full_fs_path)
+        try:
+            return hg.repository(HgUI(), self._repo.full_fs_path)
+        except hg.error.RepoError as e:
+            log.error('Error looking up repo at %s: %s',
+                    self._repo.full_fs_path, e)
+            return None
 
     def init(self):
         fullname = self._setup_paths()
@@ -326,9 +332,9 @@ class HgImplementation(M.RepositoryImplementation):
     def tarball(self, commit):
         if not os.path.exists(self._repo.tarball_path):
             os.makedirs(self._repo.tarball_path)
-        path = os.path.join(self._repo.tarball_path, commit)
         archive_name = self._repo.tarball_filename(commit)
-        filename = os.path.join(self._repo.tarball_path, '%s%s' % (archive_name, '.tar.gz'))
+        path = os.path.join(self._repo.tarball_path, archive_name)
+        filename = os.path.join(self._repo.tarball_path, '%s%s' % (archive_name, '.zip'))
         tmpfilename = os.path.join(self._repo.tarball_path, '%s%s' % (archive_name, '.tmp'))
         if os.path.exists(path):
             rmtree(path)
@@ -340,8 +346,13 @@ class HgImplementation(M.RepositoryImplementation):
                 path,
                 rev=commit,
                 prefix='')
-            with tarfile.open(tmpfilename, "w:gz") as tar:
-                tar.add(path, arcname=archive_name)
+            with ZipFile(tmpfilename, 'w') as tarball_zip:
+                for root, dirs, files in os.walk(path):
+                    for name in files:
+                        if name != '.hg_archival.txt':
+                            file_to_zip = os.path.join(root, name)
+                            arcname = file_to_zip[len(os.path.dirname(path)):].strip('/')
+                            tarball_zip.write(file_to_zip, arcname, compress_type=ZIP_DEFLATED)
             os.rename(tmpfilename, filename)
         finally:
             rmtree(path)
@@ -349,6 +360,6 @@ class HgImplementation(M.RepositoryImplementation):
                 os.remove(tmpfilename)
 
     def is_empty(self):
-        return self._hg.heads() == ['\x00'*20]
+        return not self._hg or self._hg.heads() == ['\x00'*20]
 
 Mapper.compile_all()
